
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import optparse
import subprocess
import random

# we need to import python modules from the $SUMO_HOME/tools directory
try:
    sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
    from sumolib import checkBinary  # noqa
except ImportError:
    sys.exit(
        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")
import sumolib
import traci
import csv
import math
csvFile = 'input_for_simulation.csv'
csvData = csv.reader(open(csvFile))
net = sumolib.net.readNet("Munichmapsmall.net.xml", withInternal=True)
edges = net.getEdges()
edgeLengths= {}
for edge in edges:
    key = str(edge.getID())
    edgeLengths[key] = str(edge.getLength())

def run():
    rowNum = 0
    step = 0
    routeid = list()
    departlist = list()
    edge1 = list()
    edge2 = list()
    taxiStations = list()
    stationNumber = 0
    for row in csvData:
        if rowNum == 0:
            tags = row
            rowNum += 1
        else:
            depart = 0
            for i in range(len(tags)):
                if tags[i] == "track_id":
                    routeid.append(row[i])
                if tags[i] == "hour":
                    depart += (((int(row[i]))-1) * 60 * 60)
                elif tags[i] == "minute":
                    depart += (int(row[i]) * 60)
                if tags[i] == "start_edge":
                    edge1.append(row[i])
                if tags[i] == "stop_edge":
                    edge2.append(row[i])
            departlist.append(depart)
    traci.route.add("route1", ["331340037#0", "331340037#0"])

    while traci.simulation.getMinExpectedNumber() > 0:
        traci.simulationStep()
        listOfVehicles = traci.vehicle.getIDList()  # number of vehicles running in the simulation
        if step==0:
            carNumber = 0
            for n in range(10):
                carName = "car" + str(carNumber)
                traci.vehicle.add(carName, "route1", typeID="taxi_at_the_station")
                length = float(edgeLengths["331340037#0"])
                stop =length / 2
                traci.vehicle.setStop(carName, "331340037#0", pos=stop, flags=1, duration=90000000)
                carNumber += 1
                listOfVehicles = traci.vehicle.getIDList()              #number of vehicles running in the simulation
        for k in range(len(departlist)):
                    if step==departlist[k]:
                        carIDCount = list()
                        travelTimesList = list()
                        count=0
                        for j in listOfVehicles:                                                                                    #for each vehicle we check the route, j is the vehicle ID
                                if traci.vehicle.getTypeID(j)=="taxi_at_the_station":
                                        carIDCount.append(j)                                                                       #to every car considered, we save its name in a new vector so that we can easily retrieve it back when we check the minimum
                                        currentEdge = traci.vehicle.getRoadID(j)                              #gets the ID of the current edge
                                        if currentEdge == '':
                                            currentEdge = '89840533#2'
                                        edgelist = traci.simulation.findRoute(currentEdge, edge1[k], vtype="taxi_at_the_station", depart=step, routingMode=0)
                                        travelTimesList.append(edgelist.travelTime)
                                        count+=1
                        if travelTimesList == []:                           #when no vehicle is available at the moment
                            departlist[k]= departlist[k] + 60
                            continue
                        min = int(travelTimesList[0])
                        carIDMin = carIDCount[0]
                        for x in range(len(travelTimesList)):
                            if int(travelTimesList[x])<min:
                                min=travelTimesList[x]
                                carIDMin=carIDCount[x]
                        stoppingEdge = traci.vehicle.getRoadID(carIDMin)
                        traci.vehicle.setType(carIDMin, "taxi_going_to_customer")           #type is change to signal that it is busy
                        if traci.vehicle.isStoppedParking(carIDMin):
                            traci.vehicle.resume(carIDMin)
                        traci.vehicle.changeTarget(carIDMin,edge2[k])
                        traci.vehicle.setVia(carIDMin,(edge1[k]))
                        traci.vehicle.rerouteEffort(carIDMin)
                        if stoppingEdge==edge1[k]:
                            length = float(edgeLengths[edge1[k]])
                            if length > 2:
                                stop = length
                            else:
                                route = traci.vehicle.getRoute(carIDMin)
                                pos = 0
                                while length <=2:
                                    y = len(route)
                                    for x in range(pos,y):
                                        if edge1[k]==route[x]:
                                            edge1[k]=route[x+1]
                                            pos = x+1
                                            break
                                    length = float(edgeLengths[edge1[k]])
                                stop = length
                            traci.vehicle.setStop(carIDMin, edge1[k], pos=stop, flags=0, duration=2000)
                        else:
                            length = float(edgeLengths[edge1[k]])
                            if length > 2:
                                stop = length / 2
                            else:
                                route = traci.vehicle.getRoute(carIDMin)
                                pos = 0
                                while length <=2:
                                    y = len(route)
                                    for x in range(pos,y):
                                        if edge1[k]==route[x]:
                                            edge1[k]=route[x+1]
                                            pos = x+1
                                            break
                                    length = float(edgeLengths[edge1[k]])
                                stop = length / 2
                            traci.vehicle.setStop(carIDMin, edge1[k], pos=stop, flags=0, duration=2000)
                        if stoppingEdge==edge2[k]:
                            length = float(edgeLengths[edge2[k]])
                            if length > 3:
                                stop = 1
                            else:
                                route = traci.vehicle.getRoute(carIDMin)
                                y = len(route)
                                while length <= 3:
                                    edge2[k] = route[y - 1]
                                    y -= 1
                                    length = float(edgeLengths[edge2[k]])
                                    stop = 1
                            traci.vehicle.setStop(carIDMin, edge2[k], pos=stop, flags=1, duration=90000000)
                        else:
                            length = float(edgeLengths[edge2[k]])
                            if length > 2:
                                stop = length / 2
                            else:
                                route = traci.vehicle.getRoute(carIDMin)
                                y = len(route)
                                while length <=2:
                                    edge2[k]=route[y-1]
                                    y -=1
                                    length = float(edgeLengths[edge2[k]])
                                    stop = length / 2
                            traci.vehicle.setStop(carIDMin, edge2[k], pos=stop, flags= 1, duration=90000000)
        if step > 600:
            for j in listOfVehicles:
                if traci.vehicle.getTypeID(j) == "taxi_at_the_station" or traci.vehicle.getTypeID(j)=="taxi_going_to_customer" and traci.vehicle.isStoppedParking(j):
                    traci.vehicle.remove(j, reason=3)
        step +=1
    traci.close()
    sys.stdout.flush()


def get_options():
    optParser = optparse.OptionParser()
    optParser.add_option("--nogui", action="store_true",
                         default=False, help="run the commandline version of sumo")
    options, args = optParser.parse_args()
    return options


# this is the main entry point of this script
if __name__ == "__main__":
    options = get_options()

    # this script has been called from the command line. It will start sumo as a
    # server, then connect and run
    if options.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')


    traci.start([sumoBinary, "-c", "bus.sumocfg", "--tripinfo-output", "tripinfo.xml", "--ignore-route-errors"])
    run()
